﻿--[[---------------------------------------------------------------------------
  ColaMachine, a module for sharing data between ColaLight users
  
  Copyright (C) 2006  Murazorz
  
  Part of the code is influenced by VendorValues Copyright by Saeris
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
  02110-1301, USA.
  
-------------------------------------------------------------------------------

  This is a data sharing module for ColaLight. It sends the ColaLight database
  to a channel at logon, and any new data gathered, and adds any new data
  received through the same channel to the ColaLight database.

  Any bugs, questions and requests can be posted in the 
  http://www.wowace.com/forums/index.php?topic=2351.0 thread.
-----------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("ColaMachine")

local tonumber = tonumber
local tostring = tostring
local strgmatch = string.gmatch
local strformat = string.format
local strlen = string.len

L:RegisterTranslations("enUS", function()
	return {
        ["ColaMachine"] = true,
        ["ColaMachine - blows ColaLight cans to pieces"] = true,
        ["trust"] = true,
        ["Add trusted user"] = true,
        ["<name>, <<guild>> adds all in the guild to your trust list"] = true,
        ["delete"] = true,
        ["Delete trusted user"] = true,
        ["<name>, <<guild>> deletes all in the guild from your trust list"] = true,
        ["list"] = true,
        ["List trusted users"] = true,
        ["channel"] = true,
        ["<channel>, <<default>> sets the channel to the default channel, <<addon>> sets the channel to the built-in add on channel"] = true,
        ["Set the data synchronization channel"] = true,
        ["Added the whole guild to the trusted user list"] = true,
        ["Added trusted user: "] = true,
        ["Removed the whole guild from the trusted user list"] = true,
        ["Removed trusted user: "] = true,
        ["Trusting the whole guild"] = true,
        ["Trusted Users: "] = true,
        ["Using the built-in add on channel"] = true,
        ["Setting data synchronization channel to default: "] = true,
        ["Removing use of add on channel"] = true,
        ["Setting data synchronization channel to the built-in add on channel"] = true,
        ["You are not in a guild, cannot use the built-in add on channel"] = true,
        ["Setting data synchronization channel to: "] = true,
        ["Couldn't join the proper channel for data synchronization"] = true,
	}
end)

ColaMachine = ColaLight:NewModule("ColaMachine", "AceEvent-2.0", "AceConsole-2.0")

ColaMachine.latestId = nil

local PARSING_PATTERN = ("(%d+)=.-6:([^@]*)@.->?")
local FORMATING_PATTERN = ("%s>%d=6:%d@")
local DEFAULT_CHANNEL_NAME = "lootlinkdatasharing"
local ADDON_MESSAGE_PREFIX = "VendorValueSync"

--[[--------------------------------------------------------------------------------
    OnInitialize, initializes the mod
-----------------------------------------------------------------------------------]]
function ColaMachine:OnInitialize()
    self.db = ColaLight:AcquireDBNamespace("ColaMachine")
    ColaLight:RegisterDefaults("ColaMachine", "account", {
        TrustedUsers = {},
        TrustGuild = false,
        ChannelName = DEFAULT_CHANNEL_NAME,
        UseAddonChannel = false,
    })
    
    ColaLight.options.args.ColaMachine = {
        name = L["ColaMachine"],
        desc = L["ColaMachine - blows ColaLight cans to pieces"],
        type = "group",
        args = {
            trust = {
                name = L["trust"], type = "text",
                desc = L["Add trusted user"],
                get = false,
                set = function(v) self:TrustUser(v) end,
                usage = L["<name>, <<guild>> adds all in the guild to your trust list"],
            },
            delete = {
                name = L["delete"], type = "text",
                desc = L["Delete trusted user"],
                get = false,
                set = function(v) self:DeleteUser(v) end,
                usage = L["<name>, <<guild>> deletes all in the guild from your trust list"],
            },
            list = {
                name = L["list"], type = "execute",
                desc = L["List trusted users"],
                func = function() self:ListTrustedUsers() end,
            },
            channel = {
                name = L["channel"], type = "text",
                desc = L["Set the data synchronization channel"],
                get = function() if self.db.account.UseAddonChannel then
                        return L["Using the built-in add on channel"]
                    else
                        return self.db.account.ChannelName
                    end
                end,
                set = function(v) self:SetChannel(v) end,
                usage = L["<channel>, <<default>> sets the channel to the default channel, <<addon>> sets the channel to the built-in add on channel"],
            },
        },
        handler = ColaLight,
    }
    
    AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(ColaLight, ColaLight.options.args.ColaMachine)
end

--[[--------------------------------------------------------------------------------
    OnEnable, enable the mod
-----------------------------------------------------------------------------------]]
function ColaMachine:OnEnable()
    -- register for the Links add on
    self:ScheduleRepeatingEvent(self.SendData, 15, self)
    
    -- register for chat messages
    if self.db.account.UseAddonChannel then
        if IsInGuild() then
            self:RegisterEvent("CHAT_MSG_ADDON")
        else
            self.db.account.UseAddonChannel = false
            self:Print(L["You are not in a guild, cannot use the built-in add on channel"])
            if self.db.account.ChannelName == DEFAULT_CHANNEL_NAME then
                self:Print(L["Setting data synchronization channel to default: "] .. self.db.account.ChannelName)
            else
                self:Print(L["Setting data synchronization channel to: "] .. self.db.account.ChannelName)
            end
            self:RegisterEvent("CHAT_MSG_CHANNEL")
        end
    else
        self:RegisterEvent("CHAT_MSG_CHANNEL")
    end
end

--[[--------------------------------------------------------------------------------
    OnDisable, disable the mod
-----------------------------------------------------------------------------------]]
function ColaMachine:OnDisable()
    self:CancelAllScheduledEvents()
    self:UnregisterAllEvents()
end

--[[--------------------------------------------------------------------------------
    Slash command handlers
-----------------------------------------------------------------------------------]]

function ColaMachine:TrustUser(user)
    if user:lower() == "<guild>" then
        self.db.account.TrustGuild = true
        for i = 1, GetNumGuildMembers(true) do
            local name = GetGuildRosterInfo(i)
            self.db.account.TrustedUsers[name] = 1
        end
        self:Print(L["Added the whole guild to the trusted user list"])
    else
        self.db.account.TrustedUsers[user] = 1
        self:Print(L["Added trusted user: "] .. user)
    end
end

function ColaMachine:DeleteUser(user)
    if user:lower() == "<guild>" then
        self.db.account.TrustGuild = false
        for i = 1, GetNumGuildMembers(true) do
            local name = GetGuildRosterInfo(i)
            self.db.account.TrustedUsers[name] = nil
        end
        self:Print(L["Removed the whole guild from the trusted user list"])
    else
        for k, v in pairs(self.db.account.TrustedUsers) do
            if k:lower() == user:lower() then
                self.db.account.TrustedUsers[k] = nil
                self:Print(L["Removed trusted user: "] .. user)
            end
        end
    end
end

function ColaMachine:ListTrustedUsers()
    tmp = {}
    for k, v in pairs(self.db.account.TrustedUsers) do
        table.insert(tmp, k)
    end
    if self.db.account.TrustGuild then
        self:Print(L["Trusting the whole guild"])
        for i = 1, GetNumGuildMembers(true) do
            local name = GetGuildRosterInfo(i)
            for k, v in pairs(tmp) do
                if v:lower() == name:lower() then
                    table.remove(tmp, k)
                end
            end
        end
    end
    if select("#", tmp) > 0 then
        self:Print(L["Trusted Users: "] .. table.concat(tmp, ", "))
    end
    tmp = nil
end

function ColaMachine:GetChannel()
    if self.db.account.UseAddonChannel then
        return L["Using the built-in add on channel"]
    else
        return strformat(L["Using the %s channel"], self.db.account.ChannelName)
    end
end

function ColaMachine:SetChannel(channel)
    if channel:lower() == "<default>" then
        self.db.account.UseAddonChannel = false
        self.db.account.ChannelName = DEFAULT_CHANNEL_NAME
        self:RegisterEvent("CHAT_MSG_CHANNEL")
        if self:IsEventRegistered("CHAT_MSG_ADDON") then
            self:UnregisterEvent("CHAT_MSG_ADDON")
            self:Print(L["Removing use of add on channel"])
        end
        self:Print(L["Setting data synchronization channel to default: "] .. self.db.account.ChannelName)
    elseif channel:lower() == "<addon>" then
        if IsInGuild() then
            self.db.account.UseAddonChannel = true
            self:RegisterEvent("CHAT_MSG_ADDON")
            self:UnregisterEvent("CHAT_MSG_CHANNEL")
            self:Print(L["Setting data synchronization channel to the built-in add on channel"])
        else
            self:Print(L["You are not in a guild, cannot use the built-in add on channel"])
        end
    else
        self.db.account.UseAddonChannel = false
        self.db.account.ChannelName = channel
        self:RegisterEvent("CHAT_MSG_CHANNEL")
        if self:IsEventRegistered("CHAT_MSG_ADDON") then
            self:UnregisterEvent("CHAT_MSG_ADDON")
            self:Print(L["Removing use of add on channel"])
        end
        self:Print(L["Setting data synchronization channel to: "] .. self.db.account.ChannelName)
    end
end

--[[--------------------------------------------------------------------------------
    Event Handlers
-----------------------------------------------------------------------------------]]

-- sends a package of data, 
function ColaMachine:SendData()
    local tmp, message, id, value = "", nil, next(ColaLight.db.account.SellValues, self.latestId)
    
    if id then
        local channelNumber = 0
        -- get the proper channel
        if not self.db.account.UseAddonChannel then
            channelNumber = GetChannelName(self.db.account.ChannelName)
            if channelNumber == 0 then
                channelNumber = JoinChannelByName(self.db.account.ChannelName)
                if channelNumber == 0 then
                    self:Print(L["Couldn't join the proper channel for data synchronization"])
                    return
                end
            end
        end

        local previous
        local maxlen = (self.db.account.UseAddonChannel and 254 - strlen(ADDON_MESSAGE_PREFIX)) or 255
        -- if using SendAddonMessage() the whole prefix + message can only be 254, otherwise we
        -- get disconnected, stupid blizzard
        while strlen(tmp) <= maxlen and id do
            message = tmp
            tmp = strformat(FORMATING_PATTERN, tmp, id, value)
            previous = self.latestId
            self.latestId = id
            id, value = next(ColaLight.db.account.SellValues, id)
        end
        if strlen(tmp) > 254 then
            self.latestId = previous
        end
        
        if message and message ~= "" then
            local clearAFK = GetCVar("autoClearAFK")
            SetCVar("autoClearAFK", 0)
            
            if self.db.account.UseAddonChannel then
                SendAddonMessage(ADDON_MESSAGE_PREFIX, message, "GUILD")
            else
                SendChatMessage(message, "CHANNEL", nil, channelNumber)
            end
            
            SetCVar("autoClearAFK", clearAFK)
        end
    end
end

local player = UnitName("player")
local lastMessage = ""

-- check every chat message for data, add new values if appropriate
function ColaMachine:CHAT_MSG_CHANNEL()
    local message, author, channelName = arg1, arg2, arg9
    
    -- if the message is from the right channel, not the same as the last and not from myself, 
    -- parse it and insert the price info into ColaLight
    if channelName:lower() == self.db.account.ChannelName and 
       message ~= lastMessage and 
       author:lower() ~= player:lower() then

        for id, value in strgmatch(message, PARSING_PATTERN) do
            id = tonumber(id)
            -- only add the item if we don't already have it or if it's from a trusted source
            if ColaLight.db.account.SellValues[id] == nil or self.db.account.TrustedUsers[author.lower()] then
                ColaLight.db.account.SellValues[tostring(id)] = tonumber(value)
            end
        end
    end
end

function ColaMachine:CHAT_MSG_ADDON(prefix, message, distribution, author)
    -- if the message is from ColaMachine and not from myself, 
    -- parse it and insert the price info into ColaLight
    if prefix == ADDON_MESSAGE_PREFIX and distribution == "GUILD" and author:lower() ~= player:lower() then
        for id, value in strgmatch(message, PARSING_PATTERN) do
            id = tonumber(id)
            -- only add the item if we don't already have it or if it's from a trusted source
            if ColaLight.db.account.SellValues[id] == nil or self.db.account.TrustedUsers[author.lower()] then
                ColaLight.db.account.SellValues[tostring(id)] = tonumber(value)
            end
        end
    end
end
